package org.vraptor.mydvds.dao;

import org.apache.log4j.Logger;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.Transaction;

/**
 * Fábrica de daos.
 *
 * Serve para criar daos e encapsular o acesso à camada de modelo.
 *
 * Também dar acesso a transação e sessão do Hibernate.
 */
public class DaoFactory {

	private static final Logger LOG = Logger.getLogger(DaoFactory.class);

	//a sessão atual do hibernate
	private final Session session;

	//a transação atual do hibernate
	private Transaction transaction;

	/*
	 * Construtor que recebe uma sessão do hibernate.
	 */
	public DaoFactory(Session session) {
		this.session = session;
	}

	/**
	 * Fecha a sessão.
	 */
	public void close() {
		if (this.session.isOpen()) {
			try {
				if (this.hasTransaction()) {
					this.rollback();
				}
			} catch (HibernateException e) {
				LOG.fatal("exception rolling back", e);
			} finally {
				this.session.close();
			}
		}
	}

	/**
	 * Iniciar uma transação.
	 */
	public void beginTransaction() {
		if (this.hasTransaction()) {
			this.rollback();
			throw new IllegalStateException(
					"There is no nested transaction support!");
		}
		this.transaction = this.session.beginTransaction();
	}

	/**
	 *  Executa todas as "operações" da sessão.
	 */
	public void flush() {
		this.session.flush();
	}

	/**
	 * Comita.
	 */
	public void commit() {
		this.session.flush();
		this.transaction.commit();
		this.transaction = null;
	}

	/**
	 *	Desfaz a trancação.
	 */
	public void rollback() {
		this.transaction.rollback();
		this.transaction = null;
	}

	/**
	 * Existe uma transação?
	 * @return true, se existe uma transação
	 */
	public boolean hasTransaction() {
		return this.transaction != null && this.transaction.isActive();
	}

	/**
	 * @return sessão do hibernate
	 */
	public Session getSession() {
		return session;
	}

	/**
	 * Devolve o dao de usuários.
	 *
	 * @return dao de usuários
	 */
	public UserDao getUserDao() {
		return new UserDao(getSession());
	}

	/**
	 * Devolve o dao de dvd.
	 *
	 * @return dao de dvd.
	 */
	public DvdDao getDvdDao() {
		return new DvdDao(getSession());
	}

}
