package org.vraptor.mydvds.logic;

import org.vraptor.annotations.Component;
import org.vraptor.annotations.InterceptedBy;
import org.vraptor.annotations.Logic;
import org.vraptor.annotations.Out;
import org.vraptor.i18n.Message;
import org.vraptor.mydvds.dao.DaoFactory;
import org.vraptor.mydvds.interceptor.DaoInterceptor;
import org.vraptor.mydvds.model.User;
import org.vraptor.scope.ScopeType;
import org.vraptor.validator.ValidationErrors;

/**
 * A classe Home, nome do componente "login". Será interceptada pelo <code>DaoInterceptor</code>.
 * 
 * <code>@Component</code> não especifica nenhum nome, então aplica a regra para o nome da lógica:
 * nome da classe igual nome da classe com letras minúscúlas tirando Component, Logic, Action ou Command
 * LoginComponent >>> login
 */
@Component
@InterceptedBy(DaoInterceptor.class)
public class LoginComponent {

	/*
	 * Ejeta o usuário currentUser para a sessão com key "currentUser".
	 * 
	 * É igual: httpSession.setAttribute("currentUser",currentUser)
	 */
	@Out(scope = ScopeType.SESSION)
	private User currentUser;

	private DaoFactory factory;

	/**
	 * Cria o componente e injeta a fábrica de daos pelo construtor.
	 * 
	 * Podemos usar injeção pelo construtor por causa do  
	 * <code>DaoInterceptor.class</code> que criar e ejetar a fábrica.
	 * 
	 * @param factory
	 */
	public LoginComponent(DaoFactory factory) {
		this.factory = factory;
	}

	/**
	 * URL: 			login.execute.logic
	 * View ok:			login.execute.ok 
	 * 
	 * >>> view sobrescrito no views.properties para user.home.logic:
	 *  
	 * login.execute.ok = redirect:user.home.logic
	 * 
	 * Método recebe dois parâmetros que vem do request: login e password.
	 * Como os dois sao do mesmo tipo String, temos que usar a anotação @Logic 
	 * para determinar qual parâmetro do request será usado para os parâmetros do método.
	 * 
	 * @return "ok", poderia ser "void" pq "ok" é o padrão do "void"  
	 */
	@Logic
	public String execute(String login, String password) {
		//a signatura do método poderia ser "void" 
		//pq "ok" é o padrão do "void"  
		return "ok";
	}
	
	/**
	 * Validação com VRaptor. Esse método será executado antes do método execute().
	 * VRaptor passa automaticamente um objeto do típo <code>ValidationErrors</code>.
	 * 
	 * Se uma <code>Mensagem</code> for adicionada nos <code>errors</code>, VRaptor vai redirecionar a requisição para:
	 * login.execute.invalid. Só quando os <code>errors</code> ficam vazios, o método execute() será executado.
	 * 
	 * Além do ValidationErrors, VRaptor injeta os mesmos parâmtros que são usados no método execute(...,...) 
	 * 
	 * View: 	login.execute.invalid  
	 * 
	 * >>> view sobrescrito no views.properties para index.jsp: 
	 * 
	 * login.execute.invalid = index.jsp
	 * 
	 * @param errors os erros de validação 
	 */
	public void validateExecute(ValidationErrors errors, String login, String password) {
		//procura o usuario no banco e dados
		this.currentUser = factory.getUserDao().search(login, password);
		//senão encontra nenhum usuário, adiciona uma Mensagem nos errors,
		//"invalid_login_or_password" e a chave do arquivo messages.properties,
		//a chave(key) é usado com o taglib fmt no arquivo index.jsp, por exemplo: <fmt:message key="erro.key">
		if (this.currentUser == null) {
			errors.add(new Message("login","Invalid login or password"));
		}
	}
	
	public void index() {
		//para chamar index.jsp
	}
	
}
