package org.vraptor.mydvds.logic;

import org.vraptor.annotations.Component;
import org.vraptor.annotations.InterceptedBy;
import org.vraptor.annotations.Out;
import org.vraptor.i18n.Message;
import org.vraptor.mydvds.dao.DaoFactory;
import org.vraptor.mydvds.interceptor.DaoInterceptor;
import org.vraptor.mydvds.interceptor.TransactionInterceptor;
import org.vraptor.mydvds.model.User;
import org.vraptor.plugin.hibernate.Validate;
import org.vraptor.scope.ScopeType;
import org.vraptor.validator.ValidationErrors;

/**
 * A classe NewUser, nome do componente "user". Será interceptada pelo <code>DaoInterceptor</code> e
 * <code>TransactionInterceptor</code> .
 */
@Component("user")
@InterceptedBy({DaoInterceptor.class,TransactionInterceptor.class})
public class NewUserComponent {

	private final DaoFactory factory;

	/*
	 * O atributo user é somente necessário por causa do metodo getUser (veja também o metodo add(User user)).
	 */
	private User user;

	/**
	 * Cria o componente e injeta a fábrica de daos pelo construtor.
	 *
	 * Podemos usar injeção pelo construtor por causa do
	 * <code>DaoInterceptor.class</code> que cria e ejeta a fábrica.
	 *
	 * @param factory fábrica de daos
	 */
	public NewUserComponent(DaoFactory factory) {
		this.factory = factory;
	}


	/**
	 * URL: 			user.add.logic
	 * View ok:			user.add.ok (user/add.ok.jsp)
	 *
	 * O parâmetro "user" do metodo add será preenchido com os parâmetros da requisição, por exemplo:
	 *
	 * 		user.add.logic?user.name="Nico"&user.login="555555"
	 *
	 * preenche automaticamente o parâmetro name e login do user com os valores Nico e 555555
	 *
	 * <code>@Validate(params = "user")</code> é uma anotação do Hibernate Validator Plugin e indica
	 * que o parametro "user" deve ser verificado usando o Hibernate Validator. As regras da validação
	 * estão anotados na classe <code>User</code> (veja também o tutorial do Hibernate Validator Plugin).
	 *
	 * O método adiciona o usuário no banco de dados.
	 */
	@Validate(params = "user")
	public void add(User user) {
		//adiciona usuário
		this.factory.getUserDao().add(user);
		//vamos setar o atributo da classe para que o getUser() pode ejetar o usuario
		this.user = user;
	}

	/**
	 * Validação com VRaptor. Esse método será executado antes do método add().
	 * VRaptor passa automaticamente um objeto do típo <code>ValidationErrors</code>.
	 *
	 * Se uma <code>Mensagem</code> for adicionada nos <code>errors</code>, VRaptor vai redirecionar a requisição para:
	 * user.add.invalid. Só quando os <code>errors</code> ficam vazios, o método add() será executado.
	 *
	 * View: 	user.add.invalid
	 *
	 * >>> view sobrescrito no views.properties para index.jsp:
	 *
	 * user.add.invalid = index.jsp
	 *
	 * @param errors os erros de validação
	 */
	public void validateAdd(ValidationErrors errors, User user) {
		//verifique se já existe um usuário com o login
		if (this.factory.getUserDao().containsUserWithLogin(user.getLogin())) {
			//adiciona uma mensagem de erro
			errors.add(new Message("login", "login_already_exists"));
		}
	}

	/**
	 * Ejeção do usuário pelo getter. Podemos acessar esse método no view: ${user.name}
	 *
	 * Esse é um escopo especial (FLASH). 
	 * Ele vive até o fim da proxima requisição. 
	 * Muito util para redirecionamentos, quando queremos manter os parametros durante eles.
	 *
	 * @return o usuário
	 */
	@Out(scope=ScopeType.FLASH)
	public User getUser() {
		return user;
	}
}
