package org.vraptor.mydvds.logic;

import java.util.List;

import org.vraptor.annotations.Component;
import org.vraptor.annotations.InterceptedBy;
import org.vraptor.annotations.Parameter;
import org.vraptor.annotations.Remotable;
import org.vraptor.mydvds.dao.DaoFactory;
import org.vraptor.mydvds.interceptor.SystemUserInterceptors;
import org.vraptor.mydvds.model.Dvd;

/**
 * A classe SearchDvd, nome do componente "dvd". Será interceptada pelos interceptodares definidos
 * na classe <code>SystemUserInterceptors</code>. <code>SystemUserInterceptors</code> é uma pilha
 * de interceptadores que serve para agrupar interceptadores.
 */
@Component("dvd")
@InterceptedBy(SystemUserInterceptors.class)
public class SearchDvdComponent {

	private DaoFactory factory;

	/*
	 * Ejetamos essa lista pelo getter.
	 * Sem getter @Out seria necessário para ejetar a lista.
	 */
	private List<Dvd> dvds;

	/**
	 * Cria o componente e injeta a fábrica de daos pelo construtor.
	 *
	 * Podemos usar injeção pelo construtor por causa do
	 * <code>DaoInterceptor.class</code> que cria e ejeta a fábrica.
	 * @param factory
	 */
	public SearchDvdComponent(DaoFactory factory) {
		this.factory = factory;
	}

	/**
	 * URL: 		dvd.search.logic
	 * URL ajax: 	dvd.search.ajax.logic (ejeta json)
	 * URL ajax: 	dvd.search.xml.logic  (ejeta xml)
	 * View:		dvd.search.ok (dvd/search.ok.jsp)
	 *
	 * O método procura os dvds pelo titulo.
	 *
	 * Ele usa <code>@Remotable</code> para indicar que está acessível pelas chamadas ajax.
	 *
	 * O parâmetro <code>dvd</code> do método é o parâmetro da requisição. Por exemplo o URL:
	 * 	dvd.search.ajax.logic?dvd.title="Rambo"
	 * passa um objeto dvd com o titulo "Rambo" como argumento.
	 *
	 * O uso do parâmetro dvd é uma alternativa da anotação <code>@Parameter</code>.
	 *
	 * @param dvd o parâmetro dvd do request
	 */
	@Remotable
	public void search(Dvd dvd) {

		if (dvd.getTitle() == null) {
			dvd.setTitle("");
		}
		//procura os dvds com um titulo
		this.dvds = this.factory.getDvdDao().searchSimilarTitle(dvd.getTitle());
	}

	/**
	 * URL: 		dvd.searchCombo.logic
	 * View:		dvd.searchCombo.ok (dvd/searchCombo.ok.jsp)
	 *
	 * Methodo para o plugin do jquery "autocomplete".
	 *
	 * @param dvd
	 */
	public void searchCombo(Dvd dvd) {

		if (dvd == null || dvd.getTitle() == null) {
			return;
		}
		//procura os dvds com um titulo
		this.dvds = this.factory.getDvdDao().searchSimilarTitle(dvd.getTitle());
	}


	/**
	 * Ejeção da lista de dvds pelo getter. Podemos acessar esse método no view dvd/search.ok.jsp,
	 * por exemplo: <c:forEach var="dvd" items="${dvds}">${dvd.title}</c:forEach>
	 *
	 * A lista será ejetado para o request escopos por padrão. Para demais escopos usa @Out(scope=ScopeType.....);
	 *
	 * É igual: httpRequest.setAttribute("dvds",dvds)
	 *
	 * @return a lista de Dvds
	 */
	public List<Dvd> getDvds() {
		return dvds;
	}

}
