package org.vraptor.mydvds.logic;

import java.util.ArrayList;
import java.util.List;

import org.vraptor.annotations.Component;
import org.vraptor.annotations.InterceptedBy;
import org.vraptor.annotations.Remotable;
import org.vraptor.mydvds.dao.DaoFactory;
import org.vraptor.mydvds.interceptor.AuthorizationInterceptor;
import org.vraptor.mydvds.interceptor.DaoInterceptor;
import org.vraptor.mydvds.model.User;

/**
 * A classe ShowUsersComponent, nome do componente "user". 
 * 
 * A classe será interceptada pelo <code>DaoInterceptor</code> e 
 * <code>AuthorizationInterceptor</code> 
 */
@Component("user")
@InterceptedBy( { DaoInterceptor.class, AuthorizationInterceptor.class })
public class ShowUsersComponent {
	
	private DaoFactory factory;

	/*
	 * Ejetamos essa lista pelo getter,
	 * Sem getter @Out seria necessário para ejetar ela
	 */
	private List<User> users;

	/**
	 * Cria o componente e injeta a fábrica de daos pelo construtor.
	 * 
	 * Podemos usar injeção pelo construtor por causa do  
	 * <code>DaoInterceptor.class</code> que cria e ejeta a fábrica.
	 * 
	 * @param factory fábrica de daos
	 */
	public ShowUsersComponent(DaoFactory factory) {
		this.factory = factory;
	}

	/**
	 * URL: 		user.list.logic
	 * URL ajax: 	user.list.ajax.logic
	 * View:		user.list.ok (user/list.ok.jsp)
	 * 
	 * O método prenche uma lista de usuários
	 * 
	 * Ele usa <code>@Remotable</code> para indicar que está acessível pelas chamadas ajax.
	 */
	@Remotable
	public void list() {
		this.users = new ArrayList<User>();
		// search by hand example
		List<User> users = this.factory.getUserDao().listAll();
		for (User user : users) {
			User newUser = new User();
			newUser.setId(user.getId());
			newUser.setName(user.getName());
			this.users.add(newUser);
		}
	}
	
	/**
	 * Ejeção da lista de usuários pelo getter. Podemos acessar esse método no view user/list.ok.jsp,
	 * por exemplo: <c:forEach var="user" items="${users}">${user.name}</c:forEach>
	 * 
	 * A lista será ejetado para o request escopos por padrão. Para demais escopos usa @Out(scope=ScopeType.....);
	 * 
	 * É igual: httpRequest.setAttribute("users",users)
	 * 
	 * @return a lista de usuários
	 */
	public List<User> getUsers() {
		return users;
	}
	
}
