package org.vraptor.mydvds.logic;

import org.apache.log4j.Logger;
import org.vraptor.annotations.Component;
import org.vraptor.annotations.In;
import org.vraptor.annotations.InterceptedBy;
import org.vraptor.annotations.Parameter;
import org.vraptor.i18n.Message;
import org.vraptor.interceptor.MultipartRequestInterceptor;
import org.vraptor.interceptor.UploadedFileInformation;
import org.vraptor.mydvds.dao.DaoFactory;
import org.vraptor.mydvds.interceptor.AuthorizationInterceptor;
import org.vraptor.mydvds.interceptor.DaoInterceptor;
import org.vraptor.mydvds.interceptor.SystemUserInterceptors;
import org.vraptor.mydvds.interceptor.TransactionInterceptor;
import org.vraptor.mydvds.model.Dvd;
import org.vraptor.mydvds.model.User;
import org.vraptor.scope.ScopeType;
import org.vraptor.validator.StringValidation;
import org.vraptor.validator.ValidationErrors;

/**
 * A classe SingleDvdComponent, nome do componente "dvd".
 *
 * A classe será interceptada pelos interceptodares definidos na classe <code>SystemUserInterceptors</code>.
 * <code>SystemUserInterceptors</code> é uma pilha de interceptadores que serve para agrupar interceptadores.
 *
 * O <code>TransactionInterceptor</code> serve para abrir uma transação antes de executar uma lógica
 * e comitar depois da lógica.
 *
 * O <code>MultipartRequestInterceptor</code> vem com VRaptor e serve para facilitar o upload de arquivos.
 * Verifique o tutorial sobre o upload.
 */
@Component("dvd")
@InterceptedBy( {
		AuthorizationInterceptor.class,
		DaoInterceptor.class,
		TransactionInterceptor.class,
		MultipartRequestInterceptor.class })
public class SingleDvdComponent {

	private static final Logger LOG = Logger.getLogger(SingleDvdComponent.class);

	/*
	 * @Parameter lê a parâmetro da requisição com o key "dvd",
	 * por exemplo:
	 *
	 * 		dvd.add.logic?dvd.title="Rambo"&user.login="555555"
	 *
	 * preenche automaticamente o atributo title do dvd com o valore "Rambo"
	 *
	 * É necessário criar o objeto ou usa:
	 *
	 * @Parameter(create=true)
	 * private Dvd dvd;
	 */
	@Parameter
	private Dvd dvd = new Dvd();

	private DaoFactory factory;

	/*
	 * Injeção do atributo "file" do request.
	 *
	 * O UploadedFileInformation vem do MultipartRequestInterceptor. Se existe um arquivo para upload o
	 * "file" será injetado. Verifique o tutorial sobre upload de arquivos para demais informações.
	 *
	 * @In(required = false) - "required=false" indica que o file talvez não existe
	 *
	 * É igual: httpSession.getAttribute("file");
	 */
	@In(required = false)
	private UploadedFileInformation file;

	/*
	 * Injeta o atributo "currentUser" da sessão. Como o request é o escopo padrão da anotação @In,
	 * temos que definir scope = ScopeType.SESSION como escopo desejado.
	 *
	 * É igual: httpSession.getAttribute("currentUser");
	 */
	@In(scope = ScopeType.SESSION)
	private User currentUser;

	/**
	 * Cria o componente e injeta a fábrica de daos pelo construtor.
	 *
	 * Podemos usar injeção pelo construtor por causa do
	 * <code>DaoInterceptor.class</code> que cria e ejeta a fábrica.
	 *
	 * @param factory fábrica de daos
	 */
	public SingleDvdComponent(DaoFactory factory) {
		this.factory = factory;
	}

	/**
	 * URL: 			dvd.add.logic
	 * View ok:			dvd.add.ok (dvd/add.ok.jsp)
	 *
	 * O método adiciona o dvd e atualiza o usuário no banco de dados.
	 */
	public void add() {

		//existe um arquivo?
		if (file != null) {
			//logging que existe um arquivo para upload
			LOG.info("Uploaded file: " + file.getFileName());
			//normalmente faremos uma outra coisa com o arquivo, salvar por exemplo :)
		}

		//atualiza o usuário
		factory.getUserDao().refresh(currentUser);
		//adiciona o dvd no usuário
		currentUser.getDvds().add(dvd);

		//grava o dvd e atualiza o usuário no banco de dados usando transação
		factory.getDvdDao().add(dvd);
		factory.getUserDao().update(currentUser);

	}

	/**
	 * Validação com VRaptor. Esse método será executado antes do método add().
	 * VRaptor passa automaticamente um objeto do típo <code>ValidationErrors</code>.
	 *
	 * Se uma <code>Mensagem</code> for adicionada nos <code>errors</code>, VRaptor vai redirecionar a requisição para:
	 * dvd.add.invalid. Só quando os <code>errors</code> ficam vazios, o método add() será executado.
	 *
	 * View invalid:	dvd.add.invalid
	 *
	 * >>> view sobrescrito no views.properties para index.jsp:
	 *
	 * dvd.add.invalid = redirect:user.home.logic
	 *
	 * @param errors os erros de validação
	 */
	public void validateAdd(ValidationErrors errors) {

		//validação dos valores do dvd
		if (StringValidation.isBlank(dvd.getTitle())) {
			errors.add(new Message("login", "invalid_title"));
		}
		if (StringValidation.isBlank(dvd.getDescription())
				|| dvd.getDescription().length() < 6) {
			errors.add(new Message("description", "invalid_description"));
		}
		if (dvd.getType() == null) {
			errors.add(new Message("name", "invalid_type"));
		}

	}

	public void addToMyList() {

		factory.getUserDao().refresh(currentUser);
		currentUser.getDvds().add(dvd);

		factory.getUserDao().update(currentUser);
	}

	public void validateAddToMyList(ValidationErrors errors) {

		factory.getUserDao().refresh(currentUser);
		if (currentUser.getDvds().contains(dvd)) {
			errors.add(new Message("dvd", "you_already_have_this_dvd"));
		}

	}

	/**
	 * Ejeção do objeto dvd pelo getter. Podemos acessar o dvd no view dvd/add.ok.jsp,
	 * por exemplo: ${dvd.title}
	 *
	 * O objeto user será ejetado para o request escopos por padrão. Para demais escopos usa @Out(scope=ScopeType.....);
	 *
	 * @return o dvd
	 */
	public Dvd getDvd() {
		return dvd;
	}

}
